@@ -37,8 +37,8 @@
 #include <fcntl.h>
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
+#include <libnautilus-private/nautilus-desktop-background.h>
 #include <libnautilus-private/nautilus-desktop-icon-file.h>
-#include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-directory-notify.h>
 #include <libnautilus-private/nautilus-file-changes-queue.h>
 #include <libnautilus-private/nautilus-file-operations.h>
@@ -73,6 +73,8 @@ struct FMDesktopIconViewDetails
 	gulong delayed_init_signal;
 	guint reload_desktop_timeout;
 	gboolean pending_rescan;
+
+	NautilusDesktopBackground *background;
 };
 
 static void     default_zoom_level_changed                        (gpointer                user_data);
@@ -255,7 +257,9 @@ real_begin_loading (FMDirectoryView *object)
 	view = FM_DESKTOP_ICON_VIEW (object);
 
 	icon_container = get_icon_container (view);
-	nautilus_connect_desktop_background_to_file_metadata (icon_container);
+	if (view->details->background == NULL) {
+		view->details->background = nautilus_desktop_background_new (icon_container);
+	}
 
 	FM_DIRECTORY_VIEW_CLASS (fm_desktop_icon_view_parent_class)->begin_loading (object);
 }
@@ -296,6 +300,11 @@ fm_desktop_icon_view_dispose (GObject *object)
 					      fm_directory_view_update_menus,
 					      icon_view);
 
+	if (icon_view->details->background != NULL) {
+		g_object_unref (icon_view->details->background);
+		icon_view->details->background = NULL;
+	}
+
 	G_OBJECT_CLASS (fm_desktop_icon_view_parent_class)->dispose (object);
 }
 
@@ -567,11 +576,11 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	
 	gtk_widget_queue_resize (GTK_WIDGET (icon_container));
 
-	hadj = gtk_layout_get_hadjustment (GTK_LAYOUT (icon_container));
-	vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (icon_container));
+	hadj = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (icon_container));
+	vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (icon_container));
 
-	eel_gtk_adjustment_set_value (hadj, 0);
-	eel_gtk_adjustment_set_value (vadj, 0);
+	gtk_adjustment_set_value (hadj, 0);
+	gtk_adjustment_set_value (vadj, 0);
 
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (desktop_icon_view),
 					     GTK_SHADOW_NONE);
@@ -640,9 +649,9 @@ action_change_background_callback (GtkAction *action,
 
 	nautilus_launch_application_from_command (gtk_widget_get_screen (GTK_WIDGET (data)),
 						  _("Background"),
-						  "gnome-appearance-properties",
+						  "gnome-control-center",
 						  FALSE,
-						  "--show-page=background", NULL);
+						  "background", NULL);
 }
 
 static void
