@@ -33,19 +33,14 @@
 #include "file-manager/fm-desktop-icon-view.h"
 #include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-list-view.h"
-#include "file-manager/fm-tree-view.h"
 #if ENABLE_EMPTY_VIEW
 #include "file-manager/fm-empty-view.h"
 #endif /* ENABLE_EMPTY_VIEW */
 
-#include "nautilus-application-smclient.h"
 #include "nautilus-desktop-window.h"
-#include "nautilus-history-sidebar.h"
 #include "nautilus-image-properties-page.h"
 #include "nautilus-navigation-window.h"
 #include "nautilus-navigation-window-slot.h"
-#include "nautilus-notes-viewer.h"
-#include "nautilus-places-sidebar.h"
 #include "nautilus-self-check-functions.h"
 #include "nautilus-spatial-window.h"
 #include "nautilus-window-bookmarks.h"
@@ -53,7 +48,7 @@
 #include "nautilus-window-private.h"
 #include "nautilus-window-slot.h"
 
-#include <libnautilus-private/nautilus-debug-log.h>
+#include <libnautilus-private/nautilus-dbus-manager.h>
 #include <libnautilus-private/nautilus-desktop-link-monitor.h>
 #include <libnautilus-private/nautilus-directory-private.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
@@ -65,6 +60,9 @@
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
 
+#define DEBUG_FLAG NAUTILUS_DEBUG_APPLICATION
+#include <libnautilus-private/nautilus-debug.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -279,6 +277,7 @@ ck_session_proxy_signal_cb (GDBusProxy *proxy,
 
 	if (g_strcmp0 (signal_name, "ActiveChanged") == 0) {
 		g_variant_get (parameters, "(b)", &application->session_is_active);
+		DEBUG ("ConsoleKit session is active %d", application->session_is_active);
 	}
 }
 
@@ -302,6 +301,7 @@ ck_call_is_active_cb (GDBusProxy   *proxy,
 	}
 
 	g_variant_get (variant, "(b)", &application->session_is_active);
+	DEBUG ("ConsoleKit session is active %d", application->session_is_active);
 
 	g_variant_unref (variant);
 }
@@ -363,6 +363,8 @@ ck_get_current_session_cb (GDBusConnection *connection,
 
 	g_variant_get (variant, "(&o)", &session_path);
 
+	DEBUG ("Found ConsoleKit session at path %s", session_path);
+
 	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
 				  G_DBUS_PROXY_FLAGS_NONE,
 				  NULL,
@@ -502,7 +504,7 @@ open_window (NautilusApplication *application,
 {
 	GFile *location;
 	NautilusWindow *window;
-	gboolean existing;
+	gboolean existing, open_in_browser;
 
 	if (uri == NULL) {
 		location = g_file_new_for_path (g_get_home_dir ());
@@ -510,10 +512,14 @@ open_window (NautilusApplication *application,
 		location = g_file_new_for_uri (uri);
 	}
 
+	open_in_browser = browser_window ||
+		g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
+
+	DEBUG ("Opening new window at uri %s, browser %d", uri, open_in_browser);
+
 	existing = FALSE;
 
-	if (browser_window ||
-	    g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+	if (open_in_browser) {
 		window = nautilus_application_create_navigation_window (application,
 									startup_id,
 									screen);
@@ -666,16 +672,14 @@ nautilus_application_create_desktop_windows (NautilusApplication *application)
 	NautilusDesktopWindow *window;
 	GtkWidget *selection_widget;
 	int screens, i;
-	gboolean exit_with_last_window;
 
 	display = gdk_display_get_default ();
 	screens = gdk_display_get_n_screens (display);
 
-	exit_with_last_window =
-		g_settings_get_boolean (nautilus_preferences,
-					NAUTILUS_PREFERENCES_EXIT_WITH_LAST_WINDOW);
-
 	for (i = 0; i < screens; i++) {
+
+		DEBUG ("Creating a desktop window for screen %d", i);
+		
 		selection_widget = get_desktop_manager_selection (display, i);
 		if (selection_widget != NULL) {
 			window = nautilus_desktop_window_new (application,
@@ -696,13 +700,8 @@ nautilus_application_create_desktop_windows (NautilusApplication *application)
 			nautilus_application_desktop_windows =
 				g_list_prepend (nautilus_application_desktop_windows, window);
 
-			/* don't add the desktop windows to the GtkApplication hold toplevels
-			 * if we should exit when the last window is closed.
-			 */
-			if (!exit_with_last_window) {
-				gtk_application_add_window (GTK_APPLICATION (application),
-							    GTK_WINDOW (window));
-			}
+			gtk_application_add_window (GTK_APPLICATION (application),
+						    GTK_WINDOW (window));
 		}
 	}
 }
@@ -931,8 +930,8 @@ create_window (NautilusApplication *application,
 static void
 spatial_window_destroyed_callback (void *user_data, GObject *window)
 {
-	nautilus_application_spatial_window_list = g_list_remove (nautilus_application_spatial_window_list, window);
-		
+	nautilus_application_spatial_window_list =
+		g_list_remove (nautilus_application_spatial_window_list, window);
 }
 
 NautilusWindow *
@@ -944,7 +943,6 @@ nautilus_application_get_spatial_window (NautilusApplication *application,
 					 gboolean            *existing)
 {
 	NautilusWindow *window;
-	gchar *uri;
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 
@@ -956,6 +954,8 @@ nautilus_application_get_spatial_window (NautilusApplication *application,
 			*existing = TRUE;
 		}
 
+		DEBUG ("returning existing spatial window");
+
 		return window;
 	}
 
@@ -991,12 +991,8 @@ nautilus_application_get_spatial_window (NautilusApplication *application,
 	g_object_weak_ref (G_OBJECT (window), 
 			   spatial_window_destroyed_callback, NULL);
 
-	uri = g_file_get_uri (location);
-	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "present NEW spatial window=%p: %s",
-			    window, uri);
-	g_free (uri);
-	
+	DEBUG ("Creating new spatial window");
+
 	return window;
 }
 
@@ -1055,10 +1051,8 @@ nautilus_application_create_navigation_window (NautilusApplication *application,
 	}
 	g_free (geometry_string);
 
-	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "create new navigation window=%p",
-			    window);
-
+	DEBUG ("Creating a new navigation window");
+	
 	return window;
 }
 
@@ -1093,12 +1087,18 @@ mount_added_callback (GVolumeMonitor *monitor,
 {
 	NautilusDirectory *directory;
 	GFile *root;
+	gchar *uri;
 
 	if (!application->session_is_active) {
 		return;
 	}
 		
 	root = g_mount_get_root (mount);
+	uri = g_file_get_uri (root);
+
+	DEBUG ("Added mount at uri %s", uri);
+	g_free (uri);
+	
 	directory = nautilus_directory_get_existing (root);
 	g_object_unref (root);
 	if (directory != NULL) {
@@ -1152,6 +1152,7 @@ mount_removed_callback (GVolumeMonitor *monitor,
 	NautilusWindowSlot *force_no_close_slot;
 	GFile *root, *computer;
 	gboolean unclosed_slot;
+	gchar *uri;
 
 	close_list = NULL;
 	force_no_close_slot = NULL;
@@ -1161,6 +1162,11 @@ mount_removed_callback (GVolumeMonitor *monitor,
 	window_list = gtk_application_get_windows (GTK_APPLICATION (application));
 
 	root = g_mount_get_root (mount);
+	uri = g_file_get_uri (root);
+
+	DEBUG ("Removed mount at uri %s", uri);
+	g_free (uri);
+
 	/* Construct a list of windows to be closed. Do not add the non-closable windows to the list. */
 	for (node = window_list; node != NULL; node = node->next) {
 		window = NAUTILUS_WINDOW (node->data);
@@ -1266,6 +1272,8 @@ nautilus_application_finalize (GObject *object)
 		application->proxy = NULL;
 	}
 
+	nautilus_dbus_manager_stop ();
+
         G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
 }
 
@@ -1273,18 +1281,10 @@ void
 nautilus_application_quit (NautilusApplication *self)
 {
 	GApplication *app = G_APPLICATION (self);
-	gboolean exit_with_last_window;
+	GList *windows;
 
-	exit_with_last_window =
-		g_settings_get_boolean (nautilus_preferences,
-					NAUTILUS_PREFERENCES_EXIT_WITH_LAST_WINDOW);
-
-	nautilus_application_close_desktop ();
-	g_application_release (app);
-
-	if (!exit_with_last_window) {
-		g_application_release (app);
-	}
+	windows = gtk_application_get_windows (GTK_APPLICATION (app));
+	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
 }
 
 static gint
@@ -1328,21 +1328,13 @@ nautilus_application_command_line (GApplication *app,
 	gint argc = 0;
 	gchar **argv = NULL, **uris = NULL;
 	gint retval = EXIT_SUCCESS;
-	gboolean exit_with_last_window;
 
 	context = g_option_context_new (_("\n\nBrowse the file system with the file manager"));
 	g_option_context_add_main_entries (context, options, NULL);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 
-	if (!self->initialized) {
-		g_option_context_add_group (context, egg_sm_client_get_option_group ());
-	}
-
 	argv = g_application_command_line_get_arguments (command_line, &argc);
 
-	/* we need to do this here, as parsing the EggSMClient option context,
-	 * unsets this variable.
-	 */
 	autostart_id = g_getenv ("DESKTOP_AUTOSTART_ID");
 	if (autostart_id != NULL && *autostart_id != '\0') {
 		autostart_mode = TRUE;
@@ -1368,7 +1360,7 @@ nautilus_application_command_line (GApplication *app,
 	}
 	if (kill_shell && remaining != NULL) {
 		g_application_command_line_printerr (command_line, "%s\n",
-						     _("-- quit cannot be used with URIs."));
+						     _("--quit cannot be used with URIs."));
 		retval = EXIT_FAILURE;
 		goto out;
 	}
@@ -1414,9 +1406,9 @@ nautilus_application_command_line (GApplication *app,
 		no_desktop = FALSE;
 	}
 
-	exit_with_last_window =
-		g_settings_get_boolean (nautilus_preferences,
-					NAUTILUS_PREFERENCES_EXIT_WITH_LAST_WINDOW);
+	DEBUG ("Parsing command line, no_default_window %d browser %d, quit %d, "
+	       "self checks %d, no_desktop %d",
+	       no_default_window, browser_window, kill_shell, perform_self_check, no_desktop);
 
 	if (kill_shell) {
 		nautilus_application_quit (self);
@@ -1434,10 +1426,6 @@ nautilus_application_command_line (GApplication *app,
 				nautilus_application_open_desktop (self);
 			}
 
-			if (!exit_with_last_window) {
-				g_application_hold (app);
-			}
-
 			finish_startup (self, no_desktop);
 
 			/* Monitor the preference to show or hide the desktop */
@@ -1455,9 +1443,6 @@ nautilus_application_command_line (GApplication *app,
 			g_signal_connect (gtk_accel_map_get (), "changed",
 					  G_CALLBACK (queue_accel_map_save_callback), NULL);
 
-			/* Initialize SMClient and load session info if availible */
-			nautilus_application_smclient_load (self, &no_default_window);
-
 			self->initialized = TRUE;
 		}
 
@@ -1503,6 +1488,28 @@ nautilus_application_command_line (GApplication *app,
 	return retval;
 }
 
+static void
+init_css (void)
+{
+	GtkCssProvider *provider;
+	GError *error = NULL;
+
+	provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_path (provider,
+					 NAUTILUS_DATADIR G_DIR_SEPARATOR_S "nautilus.css", &error);
+
+	if (error != NULL) {
+		g_warning ("Can't parse Nautilus' CSS custom description: %s\n", error->message);
+		g_error_free (error);
+	} else {
+		gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+							   GTK_STYLE_PROVIDER (provider),
+							   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	}
+
+	g_object_unref (provider);
+}
+
 static void
 nautilus_application_startup (GApplication *app)
 {
@@ -1513,12 +1520,11 @@ nautilus_application_startup (GApplication *app)
 	 */
 	G_APPLICATION_CLASS (nautilus_application_parent_class)->startup (app);
 
+	DEBUG ("Application startup");
+
 	/* create an undo manager */
 	self->undo_manager = nautilus_undo_manager_new ();
 
-	/* initialize the session manager client */
-	nautilus_application_smclient_startup (self);
-
 	/* Initialize preferences. This is needed to create the
 	 * global GSettings objects.
 	 */
@@ -1533,23 +1539,24 @@ nautilus_application_startup (GApplication *app)
 	fm_empty_view_register ();
 #endif /* ENABLE_EMPTY_VIEW */
 
-	/* register sidebars */
-	nautilus_places_sidebar_register ();
-	fm_tree_view_register ();
-	nautilus_history_sidebar_register ();
-	nautilus_notes_viewer_register (); /* also property page */
-
 	/* register property pages */
 	nautilus_image_properties_page_register ();
 
+	/* initialize CSS theming */
+	init_css ();
+
 	/* initialize search path for custom icons */
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 					   NAUTILUS_DATADIR G_DIR_SEPARATOR_S "icons");
+
+	nautilus_dbus_manager_start ();
 }
 
 static void
 nautilus_application_quit_mainloop (GApplication *app)
 {
+	DEBUG ("Quitting mainloop");
+
 	nautilus_icon_info_clear_caches ();
  	nautilus_application_save_accel_map (NULL);
 
@@ -1576,7 +1583,7 @@ NautilusApplication *
 nautilus_application_dup_singleton (void)
 {
 	return g_object_new (NAUTILUS_TYPE_APPLICATION,
-			     "application-id", "org.gnome.Nautilus",
+			     "application-id", "org.gnome.NautilusApplication",
 			     "flags", G_APPLICATION_HANDLES_COMMAND_LINE,
 			     NULL);
 }
