@@ -35,9 +35,6 @@
 #include <libnautilus-private/nautilus-view-factory.h>
 #include <libnautilus-private/nautilus-entry.h>
 
-#include <eel/eel-preferences.h>
-
-
 static void nautilus_navigation_window_pane_init       (NautilusNavigationWindowPane *pane);
 static void nautilus_navigation_window_pane_class_init (NautilusNavigationWindowPaneClass *class);
 static void nautilus_navigation_window_pane_dispose    (GObject *object);
@@ -63,11 +60,6 @@ real_set_active (NautilusWindowPane *pane, gboolean is_active)
 
 	/* navigation bar (manual entry) */
 	nautilus_location_bar_set_active (NAUTILUS_LOCATION_BAR (nav_pane->navigation_bar), is_active);
-
-	/* if actions/menus exist, update those as well */
-	if (NAUTILUS_NAVIGATION_WINDOW (pane->window)->details->navigation_action_group) {
-		nautilus_navigation_window_pane_initialize_tabs_menu (nav_pane);
-	}
 }
 
 static gboolean
@@ -263,11 +255,7 @@ path_bar_button_released_callback (GtkWidget *widget,
 		location = nautilus_path_bar_get_path_for_button (NAUTILUS_PATH_BAR (pane->path_bar), widget);
 
 		if (event->button == 2 && mask == 0) {
-			if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
-				flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
-			} else {
-				flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
-			}
+			flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
 		} else if (event->button == 1 && mask == GDK_CONTROL_MASK) {
 			flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
 		}
@@ -296,6 +284,16 @@ path_bar_button_drag_begin_callback (GtkWidget *widget,
 			   GINT_TO_POINTER (FALSE));
 }
 
+static void
+notebook_popup_menu_new_tab_cb (GtkMenuItem *menuitem,
+				gpointer user_data)
+{
+	NautilusWindowPane *pane;
+
+	pane = NAUTILUS_WINDOW_PANE (user_data);
+	nautilus_window_new_tab (pane->window);
+}
+
 static void
 path_bar_path_set_callback (GtkWidget *widget,
 			    GFile *location,
@@ -383,6 +381,16 @@ notebook_popup_menu_show (NautilusNavigationWindowPane *pane,
 
 	popup = gtk_menu_new();
 
+	item = gtk_menu_item_new_with_mnemonic (_("_New Tab"));
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (notebook_popup_menu_new_tab_cb),
+			  pane);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup),
+			       item);
+
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup),
+			       gtk_separator_menu_item_new ());
+
 	item = gtk_menu_item_new_with_mnemonic (_("Move Tab _Left"));
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (notebook_popup_menu_move_left_cb),
@@ -467,7 +475,7 @@ notebook_popup_menu_cb (GtkWidget *widget,
 
 static gboolean
 notebook_switch_page_cb (GtkNotebook *notebook,
-			 GtkNotebookPage *page,
+			 GtkWidget *page,
 			 unsigned int page_num,
 			 NautilusNavigationWindowPane *pane)
 {
@@ -580,7 +588,7 @@ nautilus_navigation_window_pane_hide_temporary_bars (NautilusNavigationWindowPan
 		if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
 			nautilus_navigation_window_pane_set_bar_mode (pane, NAUTILUS_BAR_SEARCH);
 		} else {
-			if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+			if (!g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
 				nautilus_navigation_window_pane_set_bar_mode (pane, NAUTILUS_BAR_PATH);
 			}
 		}
@@ -592,7 +600,7 @@ nautilus_navigation_window_pane_hide_temporary_bars (NautilusNavigationWindowPan
 	if (pane->temporary_search_bar) {
 		NautilusNavigationWindow *window;
 
-		if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+		if (!g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
 			nautilus_navigation_window_pane_set_bar_mode (pane, NAUTILUS_BAR_PATH);
 		} else {
 			nautilus_navigation_window_pane_set_bar_mode (pane, NAUTILUS_BAR_NAVIGATION);
@@ -621,6 +629,7 @@ nautilus_navigation_window_pane_setup (NautilusNavigationWindowPane *pane)
 {
 	GtkWidget *hbox;
 	NautilusEntry *entry;
+	GtkSizeGroup *header_size_group;
 
 	pane->widget = gtk_vbox_new (FALSE, 0);
 
@@ -631,11 +640,10 @@ nautilus_navigation_window_pane_setup (NautilusNavigationWindowPane *pane)
 			    FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
 
-	pane->navigation_group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
-	gtk_size_group_set_ignore_hidden (pane->navigation_group, FALSE);
+	header_size_group = NAUTILUS_NAVIGATION_WINDOW (NAUTILUS_WINDOW_PANE (pane)->window)->details->header_size_group;
 
 	pane->path_bar = g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL);
-	gtk_size_group_add_widget (pane->navigation_group, pane->path_bar);
+	gtk_size_group_add_widget (header_size_group, pane->path_bar);
 	gtk_widget_show (pane->path_bar);
 
 	g_signal_connect_object (pane->path_bar, "path_clicked",
@@ -648,7 +656,7 @@ nautilus_navigation_window_pane_setup (NautilusNavigationWindowPane *pane)
 			    TRUE, TRUE, 0);
 
 	pane->navigation_bar = nautilus_location_bar_new (pane);
-	gtk_size_group_add_widget (pane->navigation_group, pane->navigation_bar);
+	gtk_size_group_add_widget (header_size_group, pane->navigation_bar);
 	g_signal_connect_object (pane->navigation_bar, "location_changed",
 				 G_CALLBACK (navigation_bar_location_changed_callback), pane, 0);
 	g_signal_connect_object (pane->navigation_bar, "cancel",
@@ -662,7 +670,7 @@ nautilus_navigation_window_pane_setup (NautilusNavigationWindowPane *pane)
 			    TRUE, TRUE, 0);
 
 	pane->search_bar = nautilus_search_bar_new ();
-	gtk_size_group_add_widget (pane->navigation_group, pane->search_bar);
+	gtk_size_group_add_widget (header_size_group, pane->search_bar);
 	g_signal_connect_object (pane->search_bar, "activate",
 				 G_CALLBACK (search_bar_activate_callback), pane, 0);
 	g_signal_connect_object (pane->search_bar, "cancel",
@@ -730,7 +738,7 @@ gboolean
 nautilus_navigation_window_pane_path_bar_showing (NautilusNavigationWindowPane *pane)
 {
 	if (pane->path_bar != NULL) {
-		return GTK_WIDGET_VISIBLE (pane->path_bar);
+		return gtk_widget_get_visible (pane->path_bar);
 	}
 	/* If we're not visible yet we haven't changed visibility, so its TRUE */
 	return TRUE;
@@ -780,7 +788,7 @@ gboolean
 nautilus_navigation_window_pane_search_bar_showing (NautilusNavigationWindowPane *pane)
 {
 	if (pane->search_bar != NULL) {
-		return GTK_WIDGET_VISIBLE (pane->search_bar);
+		return gtk_widget_get_visible (pane->search_bar);
 	}
 	/* If we're not visible yet we haven't changed visibility, so its TRUE */
 	return TRUE;
@@ -793,8 +801,8 @@ nautilus_navigation_window_pane_hide_location_bar (NautilusNavigationWindowPane
 	gtk_widget_hide(pane->location_bar);
 	nautilus_navigation_window_update_show_hide_menu_items(
 			NAUTILUS_NAVIGATION_WINDOW (NAUTILUS_WINDOW_PANE (pane)->window));
-	if (save_preference && eel_preferences_key_is_writable(NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR)) {
-		eel_preferences_set_boolean(NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR, FALSE);
+	if (save_preference) {
+		g_settings_set_boolean (nautilus_window_state, NAUTILUS_WINDOW_STATE_START_WITH_LOCATION_BAR, FALSE);
 	}
 }
 
@@ -803,8 +811,8 @@ nautilus_navigation_window_pane_show_location_bar (NautilusNavigationWindowPane
 {
 	gtk_widget_show(pane->location_bar);
 	nautilus_navigation_window_update_show_hide_menu_items(NAUTILUS_NAVIGATION_WINDOW (NAUTILUS_WINDOW_PANE (pane)->window));
-	if (save_preference && eel_preferences_key_is_writable(NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR)) {
-		eel_preferences_set_boolean(NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR, TRUE);
+	if (save_preference) {
+		g_settings_set_boolean (nautilus_window_state, NAUTILUS_WINDOW_STATE_START_WITH_LOCATION_BAR, TRUE);
 	}
 }
 
@@ -815,7 +823,7 @@ nautilus_navigation_window_pane_location_bar_showing (NautilusNavigationWindowPa
 		return FALSE;
 	}
 	if (pane->location_bar != NULL) {
-		return GTK_WIDGET_VISIBLE (pane->location_bar);
+		return gtk_widget_get_visible (pane->location_bar);
 	}
 	/* If we're not visible yet we haven't changed visibility, so its TRUE */
 	return TRUE;
